Mapping

Struct Mapping 

pub struct Mapping { /* private fields */ }
An alignment between a substring of a query sequence and a single node using a single edit operation.

This can also represent unaligned insertions.

Implementations

Source

impl Mapping

pub fn new( seq_offset: usize, handle: usize, node_len: usize, node_offset: usize, edit: Difference, ) -> Self

Creates a new mapping starting at the given position.

Panics

Will panic if the edit extends past the end of the node. Will panic if handle is gbz::ENDMARKER.

pub fn unaligned(seq_offset: usize, edit: Difference) -> Self

Creates a mapping for an unaligned insertion.

Panics

Will panic if the edit is not an insertion.

pub fn seq_interval(&self) -> &Range<usize>

Returns the aligned interval of the query sequence.

pub fn query_len(&self) -> usize

Returns the length of the query interval.

pub fn handle(&self) -> usize

Returns the handle of the target node.

Returns gbz::ENDMARKER for an unaligned insertion.

pub fn is_unaligned(&self) -> bool

Returns true if this mapping is an unaligned insertion.

pub fn node_len(&self) -> usize

Returns the length of the target node.

pub fn node_interval(&self) -> &Range<usize>

Returns the aligned interval of the target node.

pub fn target_len(&self) -> usize

Returns the length of the target interval.

pub fn edit(&self) -> &Difference

Returns the edit operation.

pub fn follows(&self, other: &Self) -> bool

Returns true this mapping follows the given one in the same node.

pub fn is_at_start(&self) -> bool

Returns true if this mapping is at the start of the target node.

pub fn is_at_end(&self) -> bool

Returns true if this mapping is at the end of the target node.