pub struct Flags { /* private fields */ }Expand description
An ad hoc bitvector for flags in AlignmentBlock.
Implementations
Sourceimpl Flags
impl Flags
Sourcepub const FLAG_PAIR_IS_NEXT: usize = 0
pub const FLAG_PAIR_IS_NEXT: usize = 0
Flag: Pair is the next fragment in the read pair.
Sourcepub const FLAG_PAIR_IS_PROPER: usize = 1
pub const FLAG_PAIR_IS_PROPER: usize = 1
Flag: The the alignment is properly paired.
Sourcepub const FLAG_HAS_MAPQ: usize = 2
pub const FLAG_HAS_MAPQ: usize = 2
Flag: A mapping quality score is present.
Sourcepub const FLAG_HAS_SCORE: usize = 3
pub const FLAG_HAS_SCORE: usize = 3
Flag: An alignment score is present.
Sourcepub const FLAG_FULL_ALIGNMENT: usize = 4
pub const FLAG_FULL_ALIGNMENT: usize = 4
Flag: This is an alignment of the entire query sequence.
We do not encode the lengths of the unaligned flanks.
Sourcepub const FLAG_EXACT_ALIGNMENT: usize = 5
pub const FLAG_EXACT_ALIGNMENT: usize = 5
Flag: This is an exact alignment.
We do not encode the difference string, matches, or edits.
Sourcepub fn new(num_alignments: usize) -> Self
pub fn new(num_alignments: usize) -> Self
Creates a new flags object with the given number of alignments. The flags are initialized to zero (false).
Sourcepub fn set(&mut self, index: usize, flag: usize, value: bool)
pub fn set(&mut self, index: usize, flag: usize, value: bool)
Sets the given flag.
Arguments
index: The index of the alignment.flag: The flag to set.value: The value to set the flag to.