GAFBaseParams

Struct GAFBaseParams 

pub struct GAFBaseParams {
    pub block_size: usize,
    pub gbwt_buffer_size: usize,
    pub reference_free: bool,
    pub store_quality_strings: bool,
    pub store_optional_fields: bool,
}
GAF-base construction parameters.

Fields

block_size: usize

Number of alignments in a block (database row).

Default: Self::BLOCK_SIZE. Note that values 0 and 1 are functionally equivalent.

gbwt_buffer_size: usize

GBWT construction buffer size in nodes.

Default: Self::GBWT_BUFFER_SIZE. Note that the buffer grows automatically if a path is longer than the initial buffer size.

reference_free: bool

Build a reference-free GAF-base that stores sequences in table Nodes.

Default: false.

store_quality_strings: bool

Store base quality strings for the alignments.

Default: true.

store_optional_fields: bool

Store unknown optional fields with the alignments.

Default: true.

Implementations

Source

impl GAFBaseParams

pub const BLOCK_SIZE: usize = 1000

Default block size in alignments.

pub const GBWT_BUFFER_SIZE: usize = 100_000_000

Default GBWT construction buffer size in nodes.

pub fn to_json(&self) -> String

Returns a JSON description of the parameters that can be stored as a tag.