pub struct EdgeIter<'a> { /* private fields */ }Expand description
An iterator over the predecessors or successors of a node.
The type of Item is (usize, Orientation).
These values encode the identifier and the orientation of the node.
Successor nodes are always listed in sorted order.
Predecessor nodes are sorted by their identifiers, but the reverse orientation is listed before the forward orientation.
Like gbz::gbz::EdgeIter, but only for edges in the subgraph.
Implementations
Sourceimpl<'a> EdgeIter<'a>
impl<'a> EdgeIter<'a>
Sourcepub fn new(parent: &'a Subgraph, record: &'a GBZRecord, flip: bool) -> Self
pub fn new(parent: &'a Subgraph, record: &'a GBZRecord, flip: bool) -> Self
Creates a new iterator over the successors of the record.
If flip is true, the iterator will flip the orientation of the successors.
This is effectively the same as listing the predecessors of the reverse orientation of the record.