PathPosition

Struct PathPosition 

pub struct PathPosition { /* private fields */ }
A path position represented in multiple ways.

Implementations

Source

impl PathPosition

pub fn seq_offset(&self) -> usize

Returns the sequence offset in bp from the start of the path.

pub fn node_id(&self) -> usize

Returns the node identifier.

pub fn orientation(&self) -> Orientation

Returns the orientation of the node.

pub fn node_offset(&self) -> usize

Returns the offset from the start of the node.

pub fn graph_pos(&self) -> GraphPosition

Returns the graph position.

pub fn handle(&self) -> usize

Returns the GBWT node identifier / handle.

pub fn gbwt_offset(&self) -> usize

Returns the offset within the GBWT record.

pub fn gbwt_pos(&self) -> Pos

Returns the GBWT position for the oriented node containing the position.