pub enum TargetPath {
Path(Vec<usize>),
StartPosition(Pos),
}Expand description
Target path in Alignment.
The path is stored either explicitly as a sequence of oriented nodes or as its starting position in the GBWT index.
This corresponds to table Nodes in crate::GAFBase.
Variants
Path(Vec<usize>)
Path as a sequence of oriented nodes (GBWT node identifiers; GAF-base node handles).
StartPosition(Pos)
Starting position in the GBWT.