pub struct PathPosition { /* private fields */ }Expand description
A path position represented in multiple ways.
Implementations
Sourceimpl PathPosition
impl PathPosition
Sourcepub fn seq_offset(&self) -> usize
pub fn seq_offset(&self) -> usize
Returns the sequence offset in bp from the start of the path.
Sourcepub fn orientation(&self) -> Orientation
pub fn orientation(&self) -> Orientation
Returns the orientation of the node.
Sourcepub fn node_offset(&self) -> usize
pub fn node_offset(&self) -> usize
Returns the offset from the start of the node.
Sourcepub fn graph_pos(&self) -> GraphPosition
pub fn graph_pos(&self) -> GraphPosition
Returns the graph position.
Sourcepub fn gbwt_offset(&self) -> usize
pub fn gbwt_offset(&self) -> usize
Returns the offset within the GBWT record.